Package: tibble
Title: Simple Data Frames
Version: 1.4.2
Authors@R: c(
    person("Kirill", "Müller", , "krlmlr+r@mailbox.org", c("aut", "cre")),
    person("Hadley", "Wickham", , "hadley@rstudio.com", "aut"),
    person("Romain", "Francois", , "romain@r-enthusiasts.com", "ctb"),
    person("RStudio", role = "cph")
    )
Description: Provides a 'tbl_df' class (the 'tibble') that provides
    stricter checking and better formatting than the traditional data frame.
License: MIT + file LICENSE
URL: http://tibble.tidyverse.org/, https://github.com/tidyverse/tibble
BugReports: https://github.com/tidyverse/tibble/issues
Depends: R (>= 3.1.0)
Imports: cli, crayon, methods, pillar (>= 1.1.0), rlang, utils
Suggests: covr, dplyr, import, knitr (>= 1.5.32), microbenchmark,
        mockr, nycflights13, rmarkdown, testthat, withr
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: yes
RoxygenNote: 6.0.1
Collate: 'add.R' 'as_tibble.R' 'check-names.R' 'compat-lazyeval.R'
        'compat-purrr.R' 'enframe.R' 'exports.R' 'glimpse.R'
        'has-name.R' 'tibble.R' 'lst.R' 'new.R' 'repair-names.R'
        'rownames.R' 'strrep.R' 'tbl-df.r' 'tibble-package.R'
        'tribble.R' 'type-sum.r' 'utils-format.r' 'utils.r' 'wrap.R'
NeedsCompilation: yes
Packaged: 2018-01-21 23:58:53 UTC; muelleki
Author: Kirill Müller [aut, cre],
  Hadley Wickham [aut],
  Romain Francois [ctb],
  RStudio [cph]
Maintainer: Kirill Müller <krlmlr+r@mailbox.org>
Repository: CRAN
Date/Publication: 2018-01-22 19:41:48 UTC
Built: R 3.5.1; x86_64-w64-mingw32; 2018-09-08 09:26:04 UTC; windows
Archs: x64
