# Generated by roxygen2: do not edit by hand

S3method("$",tbl_df)
S3method("[",tbl_df)
S3method("[[",tbl_df)
S3method("row.names<-",tbl_df)
S3method(all.equal,tbl_df)
S3method(as.data.frame,tbl_df)
S3method(as_data_frame,"NULL")
S3method(as_data_frame,data.frame)
S3method(as_data_frame,default)
S3method(as_data_frame,list)
S3method(as_data_frame,matrix)
S3method(as_data_frame,table)
S3method(as_data_frame,tbl_df)
S3method(as_tibble,"NULL")
S3method(as_tibble,data.frame)
S3method(as_tibble,default)
S3method(as_tibble,list)
S3method(as_tibble,matrix)
S3method(as_tibble,poly)
S3method(as_tibble,table)
S3method(as_tibble,tbl_df)
S3method(as_tibble,ts)
S3method(check_names_before_after,character)
S3method(check_names_before_after,default)
S3method(check_names_df,character)
S3method(check_names_df,default)
S3method(check_names_df,logical)
S3method(check_names_df,numeric)
S3method(format_v,character)
S3method(format_v,default)
S3method(format_v,list)
S3method(glimpse,data.frame)
S3method(glimpse,default)
S3method(glimpse,tbl)
S3method(is_vector_s3,Date)
S3method(is_vector_s3,POSIXct)
S3method(is_vector_s3,data.frame)
S3method(is_vector_s3,default)
S3method(is_vector_s3,difftime)
S3method(is_vector_s3,factor)
S3method(is_vector_s3,ordered)
S3method(obj_sum,POSIXlt)
S3method(obj_sum,default)
S3method(obj_sum,list)
S3method(print,tbl_df)
S3method(print,trunc_mat)
S3method(quote_n,character)
S3method(quote_n,default)
S3method(tbl_sum,default)
S3method(tbl_sum,grouped_df)
S3method(tbl_sum,tbl_df)
S3method(tbl_sum,tbl_sql)
S3method(type_sum,Date)
S3method(type_sum,POSIXt)
S3method(type_sum,data.frame)
S3method(type_sum,default)
S3method(type_sum,difftime)
S3method(type_sum,factor)
S3method(type_sum,ordered)
S3method(type_sum,tbl_df)
export(add_column)
export(add_row)
export(as.tibble)
export(as_data_frame)
export(as_tibble)
export(column_to_rownames)
export(data_frame)
export(data_frame_)
export(deframe)
export(enframe)
export(frame_data)
export(frame_matrix)
export(glimpse)
export(has_name)
export(has_rownames)
export(is.tibble)
export(is_tibble)
export(is_vector_s3)
export(knit_print.trunc_mat)
export(lst)
export(lst_)
export(obj_sum)
export(remove_rownames)
export(repair_names)
export(rownames_to_column)
export(tbl_sum)
export(tibble)
export(tibble_)
export(tribble)
export(trunc_mat)
export(type_sum)
importFrom(Rcpp,sourceCpp)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,str)
importFrom(utils,tail)
useDynLib(tibble, .registration = TRUE)
